import os

import common
import coveragego_parser
import coveragepy_parser
import report_detector


def convert_coverage_py(data, report=None):
    if report:
        codecov_files = report['coverage']
        codecov_report = report
    else:
        codecov_files = {}
        codecov_report = {"coverage": codecov_files}
    for file_name in data['files']:
        if common.valid(file_name):
            file_object = data['files'][file_name]
            total_lines = coveragepy_parser.total_lines(file_object)
            codecov_file = codecov_files[file_name] if file_name in codecov_files else None
            if codecov_file is None:
                codecov_file = {}
                codecov_files[file_name] = codecov_file
            for i in range(1, total_lines):
                if str(i) in codecov_file:
                    curr = 0 if codecov_file[str(i)] is None else codecov_file[str(i)]
                    offset = 1 if i in file_object['executed_lines'] else 0
                    codecov_file[str(i)] = curr + offset
                else:
                    codecov_file[str(i)] = 1 if i in file_object['executed_lines'] else None
                if codecov_file[str(i)] is None:
                    codecov_file[str(i)] = 0 if i in file_object['missing_lines'] else None
    return codecov_report


def convert_coverage_go(data_text, report=None):
    if report:
        codecov_files = report['coverage']
        codecov_report = report
    else:
        codecov_files = {}
        codecov_report = {"coverage": codecov_files}

    all_lines = data_text.split("\n")

    for i in range(1, len(all_lines)):
        coverage_line = all_lines[i]
        if len(coverage_line) > coveragego_parser.MINIMAL_STATS_LENGTH:
            file_stats = coverage_line.split(":")
            absolute_file_name = file_stats[0]
            report_file_name = absolute_file_name.replace(os.getcwd(), '')
            codecov_file = codecov_files[report_file_name] if report_file_name in codecov_files else None
            if codecov_file is None:
                codecov_file = {}
                codecov_files[report_file_name] = codecov_file
                total_lines = report_detector.total_lines(absolute_file_name)
                for line in range(1, total_lines + 1):
                    codecov_file[str(line)] = None

            branch_line = file_stats[1].split(",")
            line_coverage = branch_line[1]
            line = line_coverage.split(".")[0]
            hits = line_coverage.split(" ")[2]
            back = int(line_coverage.split(" ")[1])
            codecov_file[line] = int(hits)
            for i_back in range(1, back):
                codecov_file[str(int(line) - i_back)] = hits

    return codecov_report


def convert_clover(data_xml, report=None):
    if report:
        codecov_files = report['coverage']
        codecov_report = report
    else:
        codecov_files = {}
        codecov_report = {"coverage": codecov_files}

    for project in data_xml.iter('project'):
        for file in project.iter('file'):
            absolute_filename = file.attrib['name']
            file_name = absolute_filename.replace(os.getcwd(), "")
            if file_name.startswith("/"):
                file_name = file_name[1:]
            codecov_file = codecov_files[file_name] if file_name in codecov_files else None
            if codecov_file is None:
                codecov_file = {}
                codecov_files[file_name] = codecov_file
                total_lines = report_detector.total_lines(absolute_filename)
                for line in range(1, total_lines + 1):
                    codecov_file[str(line)] = None

            for line in file.iter('line'):
                codecov_file[str(line.attrib['num'])] = int(line.attrib['count'])
    return codecov_report



import json
import os

import git
import requests

url = 'https://api.codacy.com'

repo = git.Repo(os.getcwd())
master = repo.head.reference
commit = master.commit.hexsha


def send_report(reports_pack):
    headers = {
        'Content-Type': 'application/json',
        'project-token': os.getenv('CODACY_PROJECT_TOKEN'),
    }
    reports_pack_keys = reports_pack.keys()
    first_key = list(reports_pack_keys)[0]
    if len(reports_pack_keys) == 1 and len(reports_pack[first_key]) == 1:
        effective_url = f'{url}/2.0/coverage/{commit}/{first_key}?partial=false'
        print(f"- Sending Codacy report to {effective_url}")
        r = requests.post(url=effective_url, headers=headers, data=json.dumps(reports_pack[first_key][0]))
        return r.content.decode("utf-8")
    else:
        for lang in reports_pack_keys:
            effective_url = f'{url}/2.0/coverage/{commit}/{lang}?partial=true'
            print(f"- Sending Codacy report to {effective_url}")
            for report in reports_pack[lang]:
                r = requests.post(url=effective_url, headers=headers, data=json.dumps(report))
                print("- Codacy Report sent!")
                print(f"- {r.content.decode('utf-8')}")
        effective_final_url = f'{url}/2.0/commit/{commit}/coverageFinal'
        print(f"- Sending Final Codacy report to {effective_final_url}")
        final_response = requests.post(url=effective_final_url, headers=headers, json="")
        final_response.content.decode("utf-8")
    return None



import hashlib
import os

import git

import common
import coveragego_parser
import coveragepy_parser
import report_detector

repo = git.Repo(os.getcwd())
master = repo.head.reference
commit = master.commit
commitHex = commit.hexsha


def merge_coverage(existing_file, source_file):
    target = existing_file['coverage']
    source = source_file['coverage']
    for index in range(0, len(source)):
        target[index] = source[index] if target[index] is None else target[index] + source[index]
    return target


def create_coveralls_common(report):
    if report:
        coveralls_files = report['source_files']
        coveralls_report = report
    else:
        coveralls_report = {
            'repo_token': os.getenv('COVERALLS_REPO_TOKEN'),
            'service_name': 'local-ci'}
        head = {
            'id': commitHex,
            'author_name': commit.author.name,
            'author_email': commit.author.email,
            'committer_name': commit.committer.name,
            'committer_email': commit.committer.email,
            'message': commit.message
        }
        remotes = list(map(lambda remote: {'name': remote.name, 'url': remote.url}, repo.remotes))
        git_repo = {
            'head': head,
            'branch': repo.active_branch.name,
            'remotes': remotes
        }
        coveralls_report['git'] = git_repo
        coveralls_files = []
        coveralls_report['source_files'] = coveralls_files
    return coveralls_files, coveralls_report


def convert_coverage_py(data, report=None):
    coveralls_files, coveralls_report = create_coveralls_common(report)
    source_files = coveralls_report['source_files']
    for file_name in data['files']:
        if common.valid(file_name):
            file_object = data['files'][file_name]
            text_file = open(file_name, "r")
            file_content_bytes = text_file.read().encode('utf-8')
            total_lines = coveragepy_parser.total_lines(file_object) + 2
            line_coverage = [None] * total_lines
            executed_lines = file_object['executed_lines']
            for entry in executed_lines:
                line_coverage[entry - 1] = 1
            missing_lines = file_object['missing_lines']
            for entry in missing_lines:
                line_coverage[entry - 1] = 0
            source_file = {
                'name': file_name,
                'source_digest': hashlib.md5(file_content_bytes).hexdigest(),
                'coverage': line_coverage,
            }
            filter_result = list(filter(lambda file: file['name'] == file_name, coveralls_files))
            existing_file = filter_result[0] if len(filter_result) > 0 else None
            if existing_file is None:
                source_files.append(source_file)
            else:
                existing_file['coverage'] = merge_coverage(existing_file, source_file)

    return coveralls_report


def convert_coverage_go(data_text, report=None):
    coveralls_files, coveralls_report = create_coveralls_common(report)
    total_lines = 0
    total_coverage = 0
    all_lines = data_text.split("\n")
    for i in range(1, len(all_lines)):
        coverage_line = all_lines[i]
        if len(coverage_line) > coveragego_parser.MINIMAL_STATS_LENGTH:
            file_stats = coverage_line.split(":")
            absolute_file_name = file_stats[0]
            report_file_name = absolute_file_name.replace(os.getcwd(), '')
            if report_file_name.startswith("/"):
                report_file_name = report_file_name[1:]
            total_lines = report_detector.total_lines(absolute_file_name)
            filter_result = list(filter(lambda file: file['name'] == report_file_name, coveralls_files))
            current_file_object = filter_result[0] if len(filter_result) > 0 else None
            if current_file_object is None:
                line_coverage = [None] * total_lines
                text_file = open(absolute_file_name, "r")
                file_content_bytes = text_file.read().encode('utf-8')
                current_file_object = {
                    'name': report_file_name,
                    'source_digest': hashlib.md5(file_content_bytes).hexdigest(),
                    "coverage": line_coverage
                }
                coveralls_files.append(current_file_object)

            coverage_lines = current_file_object['coverage']
            branch_line = file_stats[1].split(",")
            go_line_coverage = branch_line[1]
            line = go_line_coverage.split(".")[0]
            hits = go_line_coverage.split(" ")[2]
            back = int(go_line_coverage.split(" ")[1])
            coveragego_parser.merge(coverage_lines, int(line) - 1, int(hits))
            for i_back in range(1, back):
                coveragego_parser.merge(coverage_lines, int(line) - i_back - 1, int(hits))

    return coveralls_report


def convert_clover(data_xml, report=None):
    coveralls_files, coveralls_report = create_coveralls_common(report)
    source_files = coveralls_report['source_files']
    for project in data_xml.iter('project'):
        for file in project.iter('file'):
            absolute_filename = file.attrib['name']
            if common.valid(absolute_filename):
                text_file = open(absolute_filename, "r")
                file_content_bytes = text_file.read().encode('utf-8')
                total_lines = report_detector.total_lines(absolute_filename) + 2
                line_coverage = [None] * total_lines
                for line in file.iter('line'):
                    line_coverage[int(line.attrib['num']) - 1] = int(line.attrib['count'])
                file_name = absolute_filename.replace(os.getcwd(), "")
                if file_name.startswith("/"):
                    file_name = file_name[1:]
                source_file = {
                    'name': file_name,
                    'source_digest': hashlib.md5(file_content_bytes).hexdigest(),
                    'coverage': line_coverage,
                }
                filter_result = list(filter(lambda file: file['name'] == absolute_filename, coveralls_files))
                existing_file = filter_result[0] if len(filter_result) > 0 else None
                if existing_file is None:
                    source_files.append(source_file)
                else:
                    existing_file['coverage'] = merge_coverage(existing_file, source_file)

    return coveralls_report



def total_lines(file_name):
    f = open(file_name)
    data = f.readlines()
    f.close()
    return len(data)


def is_coverage_py(report_text):
    if report_text.startswith("{") and report_text.endswith(
            "}") and "\"meta\":" in report_text and "\"files\":" in report_text:
        return True
    return False


def is_coverage_go(report_text):
    if report_text.startswith("mode:") and "go:" in report_text:
        return True
    return False


def is_clover(report_text):
    if "coverage generated" in report_text and "project timestamp" in report_text:
        return True
    return False



#!/usr/bin/python
import glob
import json
import os
import xml.etree.ElementTree as ET

import codacy_client
import codacy_converter
import codecov_client
import codecov_converter
import coveralls_client
import coveralls_converter
from report_detector import is_coverage_go, is_clover
from report_detector import is_coverage_py

banner = """
  ______   .___  ___. .__   __.  __     .______       _______ .______     ______   .______     .___________. _______ .______
 /  __  \  |   \/   | |  \ |  | |  |    |   _  \     |   ____||   _  \   /  __  \  |   _  \    |           ||   ____||   _  \ 
|  |  |  | |  \  /  | |   \|  | |  |    |  |_)  |    |  |__   |  |_)  | |  |  |  | |  |_)  |   `---|  |----`|  |__   |  |_)  |
|  |  |  | |  |\/|  | |  . `  | |  |    |      /     |   __|  |   ___/  |  |  |  | |      /        |  |     |   __|  |      /
|  `--'  | |  |  |  | |  |\   | |  |    |  |\  \----.|  |____ |  |      |  `--'  | |  |\  \----.   |  |     |  |____ |  |\  \----.
 \______/  |__|  |__| |__| \__| |__|    | _| `._____||_______|| _|       \______/  | _| `._____|   |__|     |_______|| _| `._____|

<Experimental Python Version>

by João Esperancinha
"""


def create_reports(all_report_texts):
    print(banner)
    codacy_reports = {}
    codecov_report = None
    coveralls_report = None
    for data_text in all_report_texts:
        codecov_token = os.getenv('CODECOV_TOKEN')
        if codecov_token is not None:
            print("Processing Codecov reports...")
            if is_coverage_py(data_text):
                codecov_report = codecov_converter.convert_coverage_py(json.loads(data_text), codecov_report)
            elif is_coverage_go(data_text):
                codecov_report = codecov_converter.convert_coverage_go(data_text, codecov_report)
            elif is_clover(data_text):
                codecov_report = codecov_converter.convert_clover(ET.fromstring(data_text), codecov_report)
        else:
            print("* CODECOV_TOKEN not configured.")

        coveralls_token = os.getenv('COVERALLS_REPO_TOKEN')
        if coveralls_token is not None:
            print("Processing Coveralls reports...")
            if is_coverage_py(data_text):
                coveralls_report = coveralls_converter.convert_coverage_py(json.loads(data_text), coveralls_report)
            elif is_coverage_go(data_text):
                coveralls_report = coveralls_converter.convert_coverage_go(data_text, coveralls_report)
            elif is_clover(data_text):
                coveralls_report = coveralls_converter.convert_clover(ET.fromstring(data_text), coveralls_report)
        else:
            print("* COVERALLS_REPO_TOKEN not configured.")

        codacy_token = os.getenv('CODACY_PROJECT_TOKEN')
        if codacy_token is not None:
            print("Processing Codacy reports...")
            if is_coverage_py(data_text):
                py_report = codacy_converter.convert_coverage_py(json.loads(data_text))
                python_lang = codacy_converter.Language.PYTHON.capitalized()
                if python_lang not in codacy_reports:
                    codacy_reports[python_lang] = []
                codacy_reports[python_lang].append(py_report)
            elif is_coverage_go(data_text):
                go_report = codacy_converter.convert_coverage_go(data_text)
                go_lang = codacy_converter.Language.GO.capitalized()
                if go_lang not in codacy_reports:
                    codacy_reports[go_lang] = []
                if go_report is not None:
                    codacy_reports[go_lang].append(go_report)
            elif is_clover(data_text):
                php_report = codacy_converter.convert_clover(ET.fromstring(data_text))
                php_lang = codacy_converter.Language.PHP.capitalized()
                if php_lang not in codacy_reports:
                    codacy_reports[php_lang] = []
                if php_report is not None:
                    codacy_reports[php_lang].append(php_report)
        else:
            print("* CODACY_PROJECT_TOKEN not configured.")

    if codecov_report is not None:
        print(codecov_client.send_report([json.dumps(codecov_report)]))
        print("Codecov reporting complete!")

    if coveralls_report is not None:
        print(coveralls_client.send_report(json.dumps(coveralls_report)))
        print("Coveralls reporting complete!")

    if len(codacy_reports.values()) > 0:
        print(codacy_client.send_report(codacy_reports))
        print("Codacy reporting complete!")


def get_text_from_file(file_name):
    f = open(file_name)
    data = f.read()
    f.close()
    return data


if __name__ == '__main__':
    allDocs = []
    coveragePyDocs = [name for name in glob.glob("coverage*.json")]
    coverageGoDocs = [name for name in glob.glob("coverage*.out")]
    coverageCloverDocs = [name for name in glob.glob("clover*.xml")]
    allDocs.extend(coveragePyDocs)
    allDocs.extend(coverageGoDocs)
    allDocs.extend(coverageCloverDocs)

    print(f"- Found potential report files {allDocs}")

    allDocs = map(lambda file: get_text_from_file(file), allDocs)

    create_reports(allDocs)



import os

import git
import requests

repo = git.Repo(os.getcwd())
master = repo.head.reference
commit = master.commit.hexsha

url = 'https://codecov.io/upload/v4'

separator = "\n<<<<<< EOF\n"


def send_report(codecov_reports):
    params = {
        'commit': commit,
        'token': os.getenv('CODECOV_TOKEN'),
        'branch': master
    }
    headers = {
        'Content-Type': 'text/plain'
    }
    r = requests.post(url, params=params, headers=headers, data="")
    decode = r.content.decode("utf-8").split(sep='\n')
    view = decode[0]
    print(f"- Report contents will be visible here {view}")
    upload_to_s3_url = decode[1]
    print(f"- Sending contents to {upload_to_s3_url}")
    codecov_report = separator.join(codecov_reports) + separator
    r = requests.put(upload_to_s3_url, headers=headers, data=codecov_report)
    return r.content.decode("utf-8")



# -*- coding: utf-8 -*-
from distutils.core import setup

from setuptools import find_packages

with open("README.txt", "r") as fh:
    long_description = fh.read()

setup(
    long_description=long_description,
    long_description_content_type="text/markdown",
    name='omni_coveragereporter',
    package_dir={'': 'src/omni_coveragereporter'},
    packages=find_packages(where='src/omni_coveragereporter'),
    py_modules=["omni_coveragereporter"],
    scripts=[
        'src/omni_coveragereporter/codacy_client.py',
        'src/omni_coveragereporter/codacy_converter.py',
        'src/omni_coveragereporter/codecov_client.py',
        'src/omni_coveragereporter/codecov_converter.py',
        'src/omni_coveragereporter/common.py',
        'src/omni_coveragereporter/coveragego_parser.py',
        'src/omni_coveragereporter/coveragepy_parser.py',
        'src/omni_coveragereporter/coveralls_client.py',
        'src/omni_coveragereporter/coveralls_converter.py',
        'src/omni_coveragereporter/omni_coveragereporter_python.py',
        'src/omni_coveragereporter/report_detector.py',
    ],
    version='0.0.3',
    description='Omni Coverage Reporter. Strives to be as universal as possible in receiving, converting and sending reports from many types to different online API''s',
    author='João Esperancinha',
    author_email='jofisaes@gmail.com',
    url='http://joaofilipesabinoesperancinha.nl/main',
    keywords=['report', 'codacy', 'coveralls', 'codecov'],
    install_requires=[
        # 'math',
        # 'random',
    ],
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'Topic :: Software Development :: Build Tools',
        'License :: OSI Approved :: Apache Software License',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
    ],
)



import os
from enum import Enum

import common
import coveragego_parser
import report_detector


class Language(Enum):
    JAVA = 1
    KOTLIN = 2
    SCALA = 3
    PYTHON = 4
    JAVA_SCRIPT = 5
    GO = 6
    PHP = 7

    def capitalized(self):
        name = self.name
        return "".join(list(map(lambda t: t.capitalize(), name.split("_"))))

    def ext(self):
        if self is Language.JAVA:
            return "java"
        if self is Language.KOTLIN:
            return "kt"
        if self is Language.SCALA:
            return "scala"
        if self is Language.PYTHON:
            return "py"
        if self is Language.JAVA_SCRIPT:
            return "js"
        if self is Language.GO:
            return "go"
        if self is Language.PHP:
            return "php"


def convert_coverage_py(data):
    data_files = data['files']
    total_covered = sum(map(lambda fn: len(data_files[fn]['executed_lines']), data_files))
    total_lines = sum(map(lambda fn: len(data_files[fn]['executed_lines']) + len(
        data_files[fn]['missing_lines']), data_files))
    total_percent = total_covered * 100 / total_lines
    codacy_report = {'total': int(total_percent)}
    codacy_file_reports = []
    codacy_report['fileReports'] = codacy_file_reports

    for file_name in data_files:
        if common.valid(file_name):
            file_object = data_files[file_name]
            file_covered = len(file_object['executed_lines'])
            file_lines = len(file_object['executed_lines']) + len(file_object['missing_lines'])
            codacy_file_report = {'filename': file_name, 'total': int(file_covered * 100 / file_lines)}
            line_coverage = {}
            for index in file_object['executed_lines']:
                line_coverage[str(index)] = 1
            for index in file_object['missing_lines']:
                line_coverage[str(index)] = 0
            codacy_file_reports.append(codacy_file_report)
            codacy_file_report['coverage'] = line_coverage
    return codacy_report


def convert_coverage_go(data_text):
    file_reports = []
    codacy_report = {
        'total': 0,
        'fileReports': file_reports
    }
    all_lines = data_text.split("\n")
    total_lines = 0
    total_coverage = 0

    for i in range(1, len(all_lines)):
        coverage_line = all_lines[i]
        if len(coverage_line) > coveragego_parser.MINIMAL_STATS_LENGTH:
            file_stats = coverage_line.split(":")
            absolute_file_name = file_stats[0]
            report_file_name = absolute_file_name.replace(os.getcwd(), '')
            if report_file_name.startswith("/"):
                report_file_name = report_file_name[1:]
            file_lines = report_detector.total_lines(absolute_file_name)
            total_lines += file_lines
            filter_result = list(filter(lambda file: file['filename'] == report_file_name, file_reports))
            current_file_object = filter_result[0] if len(filter_result) > 0 else None
            if current_file_object is None:
                line_coverage = {}
                for line in range(1, file_lines + 1):
                    line_coverage[str(line)] = 0
                current_file_object = {
                    'filename': report_file_name,
                    'total': 0,
                    'coverage': line_coverage
                }
                file_reports.append(current_file_object)

            coverage_lines = current_file_object['coverage']
            branch_line = file_stats[1].split(",")
            line_coverage = branch_line[1]
            line = line_coverage.split(".")[0]
            hits = line_coverage.split(" ")[2]
            back = int(line_coverage.split(" ")[1])
            coverage_lines[line] = int(hits)
            total_coverage += 1 if int(hits) > 0 else 0
            for i_back in range(1, back):
                total_coverage += 1 if int(hits) > 0 else 0
                coverage_lines[str(int(line) - i_back)] = int(hits)
    codacy_report['total'] = int((total_coverage * 100) / total_lines)
    for source in file_reports:
        coverage_per_file = source["coverage"]
        source["total"] = int(
            (len(list(filter(lambda x: x > 0, coverage_per_file.values()))) * 100) / len(coverage_per_file))
    return codacy_report


def convert_clover(data_xml):
    total_covered = 0
    total_lines = 0
    codacy_report = {'total': 0}
    codacy_file_reports = []
    codacy_report['fileReports'] = codacy_file_reports

    for project in data_xml.iter('project'):
        for file in project.iter('file'):
            absolute_filename = file.attrib['name']
            file_name = absolute_filename.replace(os.getcwd(), "")
            if file_name.startswith("/"):
                file_name = file_name[1:]
            if common.valid(file_name):
                file_covered = 0
                file_lines = report_detector.total_lines(absolute_filename)
                line_coverage = {}
                for line in file.iter('line'):
                    count = int(line.attrib['count'])
                    line_coverage[str(line.attrib['num'])] = count
                    if count > 0:
                        total_covered += 1
                total_lines += file_lines
                codacy_file_report = {'filename': file_name, 'total': int(file_covered * 100 / file_lines)}
                codacy_file_reports.append(codacy_file_report)
                codacy_file_report['coverage'] = line_coverage

    total_percent = int((total_covered * 100) / total_lines)
    codacy_report['total'] = total_percent
    return codacy_report
